@@ -4,11 +4,13 @@ pub mod bluetooth;
 pub mod cli;
 pub mod config;
 pub mod event;
+pub mod favorite;
 pub mod handler;
 mod help;
 pub mod notification;
 pub mod requests;
 pub mod rfkill;
 pub mod spinner;
+pub mod string_ref;
 pub mod tui;
 pub mod ui;
